#include <err.h>
#include <stdio.h>
#include <stdlib.h>

#define INPUT "opgave1.in"
#define OUTPUT "opgave1.out"

#define CODE 0
#define HINT 1

#define IMPOSSIBLE -1
#define INDETERMINATE -2

#define MAXHINTS 10000
#define MAXCODES 10000

/* Strategy:
 * For each hint, filter out all codes which do not match.
 * If after all hints are processed:
 *   - no codes remain it is impossible
 *   - more than code remains it is indeterminate
 *   - exactly one code remains, it is the solution
 */

/* File descriptors. */
FILE* fd_in;
FILE* fd_out;

/* All codes which are still available are connected with a linked
 * list.
 */
struct code {
	int c;
	struct code *next;
	struct code *prev;
};

/* Number of test cases in the file. */
int n;

/* Each address with the associated house number. */
int hints[MAXHINTS][2];

/* Number of `addresses'. */
int num_hints;

/* Final solution is stored in here */
int solution;

/* Pointer to the first valid pin code, can be NULL in case there isn't any
 * valid pin code.
 */

struct code *first;

/* All pincodes pincode[i] == i */
struct code pincodes[MAXCODES];

/* Given a code and a guess, it returns the corresponding hint,
 * i.e. given the pincode and the post code it returns the
 * house number.
 */
int
get_hint(int code, int guess)
{
	int i, j;
	int hint = 0;
	int c_a[4];
	int g_a[4];
	/* Every digit can only be matched one time. */
	int matched_c[4] = {0, 0, 0, 0};
	int matched_g[4] = {0, 0, 0, 0};

	/* Convert each number to an array */
	for (i = 0; i < 4; i++) {
		c_a[3 - i] = code % 10;
		g_a[3 - i] = guess % 10;
		code /= 10;
		guess /= 10;
	}

	/* First match those digits which are in
	 * the same position.
	 */
	for (i = 0; i < 4; i++) {
		if (c_a[i] == g_a[i]) {
			matched_c[i] = matched_g[i] = 1;
			hint += 10;
		}
	}

	/* Then those which aren't in the same position. */
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 4; j++) {
			if (c_a[i] == g_a[j] && !matched_c[i]
					&& !matched_g[j]) {
				matched_c[i] = 1;
				matched_g[j] = 1;
				hint += 1;
			}
		}
	}
	return hint;
}

/* Rebuilds the linked list.
 * Has to be done for every pincode.
 */
void
initialize()
{
	int i;
	first = &pincodes[0];
	for (i = 0; i < MAXCODES; i++) {
		pincodes[i].c = i;
		if (i > 0)
			pincodes[i].prev = &pincodes[i - 1];
		else
			pincodes[i].prev = NULL;
		if (i < MAXCODES - 1)
			pincodes[i].next = &pincodes[i + 1];
		else
			pincodes[i].next = NULL;
	}
}

/* For each post code and the house number it filters out all
 * elements in the linked list which do not correspond to it.
 */
void
calculatesolution()
{
	int i;
	struct code *current;
	for (i = 0; i < num_hints; i++) {
		current = first;
		while (current) {
			if (get_hint(current->c, hints[i][CODE])
					!= hints[i][HINT]) {
				/* Eliminate the item. */
				if (current->prev) {
					current->prev->next = current->next;
					/* If it isn't the final item
					 * update the next item too.
					 */
					if (current->next)
						current->next->prev
							= current->prev;
				} else {
					/* It's the first one in the list,
					 * update first.
					 * We can empty the whole list now,
					 * in that case current->next is NULL
					 */
					first = current->next;
					if (first)
						first->prev = NULL;
				} /* Else just keep the item. */
			}
			current = current->next;
		}
	}
	/* No solutions left. */
	if (first == NULL)
		solution = IMPOSSIBLE;
	/* One solution left. */
	else if (first->next == NULL)
		solution = first->c;
	/* More than one solution left. */
	else
		solution = INDETERMINATE;
}

/* Read the number of test cases. */
void
readinput_n(void)
{
	fd_in = fopen(INPUT, "r");
	if (fd_in == NULL)
		errx(EXIT_FAILURE, "could not open %s", INPUT);
	fscanf(fd_in, "%d\n", &n);
}

/* Read one test case. */
void
readinput(void)
{
	int i;
	fscanf(fd_in, "%d\n", &num_hints);
	for (i = 0; i < num_hints; i++)
		fscanf(fd_in, "%d %d", &hints[i][CODE], &hints[i][HINT]);
}

/* Close the input. */
void
closeinput(void)
{
	fclose(fd_in);
}

/* Open the output. */
void
openoutput(void)
{
	fd_out = fopen(OUTPUT, "w");
	if (fd_out == NULL)
		errx(EXIT_FAILURE, "could not open %s", OUTPUT);
}

/* Write the output for one test case */
void
writeoutput(void)
{
	if (solution == IMPOSSIBLE)
		fprintf(fd_out, "onmogelijk\n");
	else if (solution == INDETERMINATE)
		fprintf(fd_out, "onbepaald\n");
	else
		fprintf(fd_out, "%04d\n", solution);
}

/* Close the output. */
void
closeoutput(void)
{
	fclose(fd_out);
}

int
main(void)
{
	/* Read the number of test cases. */
	readinput_n();
	openoutput();
	/* Then for each test case. */
	while (n--) {
		initialize();
		readinput();
		calculatesolution();
		writeoutput();
	}
	/* And done. */
	closeinput();
	closeoutput();
	return 0;
}
