#include <err.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define INPUT "opgave5.in"
#define OUTPUT "opgave5.out"

/* Persons of each sex. */
#define MAXPERSONS 100

/* Strategy:
 * First read the input file and find out which people want to see
 * each other both.
 *
 * Using these data fill the persons array and use this for
 * the hungarian method to find a maximum matching.
 */

/* If matches[i][j] == 1 then they want to date each other.
 */
char matches[MAXPERSONS][MAXPERSONS];

/* The data structure of a person. */
struct person {
	int num;
	int sex;
	int queued;
	int visited;
	int num_neigh;
	int processed;
	struct person *neighbours[100];
	struct person *match;
	struct person *parent;
};

/* Man and woman. */
struct person persons[2*MAXPERSONS];

/* Number of test cases. */
int n;
/* Number of men. */
int num_man;
/* Number of women. */
int num_woman;
/* Number of persons, is num_man + num_woman. */
int num_persons;

/* File pointers. */
FILE* fd_in;
FILE* fd_out;

/* Returns the first M-unsaturated, unprocessed person. */
struct person*
get_unsat_unproc(void)
{
	int i;
	for (i = 0; i < num_persons; i++)
		if (persons[i].processed == 0 && persons[i].match == NULL)
			return &persons[i];
	/* Nothing found. */
	return NULL;
}

/* Sets processed back to zero, to be run after the matching is extended. */
void
unprocess_all(void)
{
	int i;
	for (i = 0; i < num_persons; i++)
		persons[i].processed = 0;
}

/* Finds an augmenting path, starting at the node p */
/* It doesn't necessarily find the longest augmenting path, but because all
 * nodes start unprocessed again anyway, this will work out. It now just stops
 * as soon as it has a path, this is good too, because otherwise the flipping
 * wouldn't work.
 *
 * This is a BFS search.
 */
int
find_augmenting_path(struct person *p, struct person *parent)
{
	int i;
	int qi = 0;
	int qp = 0;
	int found = 0; /* Start without a path. */
	struct person *node;
	struct person *q[2 * MAXPERSONS];
	
	for (i = 0; i < 2 * MAXPERSONS; i++)
		q[i] = NULL;

	p->parent = NULL;
	q[qi++] = p;

	while (q[qp] && !found) {
		/* To prevent those nasty loops. */
		q[qp]->visited = 1;
		for (i = 0; i < q[qp]->num_neigh; i++) {
			if (!q[qp]->neighbours[i]->visited
					&& !q[qp]->neighbours[i]->match) {
				/* Found a path. */
				q[qp]->neighbours[i]->match = q[qp];
				q[qp]->match = q[qp]->neighbours[i];
				node = q[qp]->parent;
				/* Flip the augmenting path. */
				while(node) {
					if (node->match->match != node) {
						node->match = node->parent;
						node->parent->match = node;
					}
					node = node->parent;
				}
				/* Succes! */
				found = 1;
				break;
			} else if (!q[qp]->neighbours[i]->visited
					&& !q[qp]->neighbours[i]->queued) {
				/* Enqueue this node. */
				q[qi] = q[qp]->neighbours[i];
				q[qi]->parent = q[qp];
				q[qi++]->queued = 1;
			}
		}
		qp++;
	}

	for (i = 0; i < num_persons; i++) {
		persons[i].visited = 0;
		persons[i].queued = 0;
	}
	return found;
}

/* An implementation of the Hungarian method. */
void
calculatesolution()
{
	struct person *p;
	/* Everything starts unprocessed. */
	unprocess_all();
	/* Matching = empty. By starting with a greedy
	 * match one can speed up the proces, but that's
	 * not needed.
	 */
	
	while ((p = get_unsat_unproc())) {
		if (find_augmenting_path(p, NULL)) {
			unprocess_all();
		} else
			/* Node remains unmatched for now. */
			p->processed = 1;
	}
	
	/* No more unsaturated un processed nodes, means the current
	 * matching is the maximal matching.
	 */
}

/* Read the number of test cases. */
void
readinput_n(void)
{
	fd_in = fopen(INPUT, "r");
	if (fd_in == NULL)
		errx(EXIT_FAILURE, "could not open %s", OUTPUT);
	fscanf(fd_in, "%d\n", &n);
}

/* Read a test case. */
void
readinput(void)
{
	/* This is enough. Too much actually */
	char line[1024];
	int n;
	int i, j;
	char* num;

	/* Clean everything. */
	memset(matches, 0, MAXPERSONS*MAXPERSONS);
	memset(persons, 0, sizeof(struct person) * MAXPERSONS * 2);

	fscanf(fd_in, "%d\n", &num_man);
	fscanf(fd_in, "%d\n", &num_woman);

	num_persons = num_man + num_woman;

	for (i = 0; i < num_man; i++) {
		fgets(line, 1024, fd_in);
		num = strtok(line, " ");
		while (num) {
			sscanf(num, "%d", &n);
			matches[i][n - 1]++;
			num = strtok(NULL, " ");
		}
	}
	for (i = 0; i < num_woman; i++) {
		fgets(line, 1024, fd_in);
		num = strtok(line, " ");
		while (num) {
			sscanf(num, "%d", &n);
			matches[n - 1][i]++;
			num = strtok(NULL, " ");
		}
	}

	/* If there is a two-way match, the value
	 * will be '2' in the array, if there's only
	 * a one way match, it'll be '1'.
	 *
	 * All 2->1 and all 1->0, and 0 remains 0.
	 */
	for (i = 0; i < num_man; i++)
		for (j = 0; j < num_woman; j++)
			if (matches[i][j])
				matches[i][j]--;
	
	/* Fill the array with person data, which makes
	 * the hungarian method a lot easier to program.
	 *
	 * The women start after the men.
	 */
	for (i = 0; i < num_man; i++) {
		for (j = 0; j < num_woman; j++)
			if (matches[i][j]) {
				persons[i].num = i;
				persons[i].sex = 0;
				persons[i].neighbours[persons[i].num_neigh] =
					&persons[num_man + j];
				persons[i].num_neigh++;
				persons[i].match = NULL;
				persons[i].queued = 0;
				persons[i].visited = 0;
					
				persons[num_man + j].num = j;
				persons[num_man + j].sex = 1;
				persons[num_man + j].neighbours[
					persons[num_man + j].num_neigh] =
					&persons[i];
				persons[num_man + j].num_neigh++;
				persons[num_man + j].match = NULL;
				persons[num_man + j].queued = 0;
				persons[num_man + j].visited = 0;
			}
	}
}

/* Close the input. */
void
closeinput(void)
{
	fclose(fd_in);
}

/* Open the output. */
void
openoutput(void)
{
	fd_out = fopen(OUTPUT, "w");
}

/* Write one test case. */
void
writeoutput(void)
{
	int i;
	int matched = 0;
	for (i = 0; i < num_persons; i++)
		if (persons[i].match)
			matched++;
	if (matched % 2 != 0)
		printf("Uh. Oh.\n");
	/* We have to report the number of couples, i.e.
	 * the number of lines in the matching, not the number
	 * of vertices.
	 */
	matched >>= 1;
	fprintf(fd_out, "%d\n", matched);
}

/* Close the output. */
void
closeoutput(void) {
	fclose(fd_out);
	if (fd_out == NULL)
		errx(EXIT_FAILURE, "could not open %s", OUTPUT);
}

	int
main(void)
{
	/* Read the number of testcases. */
	readinput_n();
	openoutput();
	/* For each testcase. */
	while (n--) {
		readinput();
		calculatesolution();
		writeoutput();
	}
	/* And done. */
	closeinput();
	closeoutput();
	return 0;
}
