#include <err.h>
#include <stdio.h>
#include <stdlib.h>

#define INPUT "opgave6.in"
#define OUTPUT "opgave6.out"

#define MAXSIZE 100
#define NUMLETTERS 26

/* Strategy:
 * For each field in the pyramid keep an array of 26 values,
 * for each letter a value, which represents for each letter
 * the number of occurences (maximal) when arriving on that position.
 *
 * This is determined of course by inspecting the arrays from the
 * neighbours to the left and the right (if applicable) and choosing
 * the maximum.
 *
 * That way, starting at the top the whole array can be filled.
 */

/* The pyramid. */
char letters[MAXSIZE][MAXSIZE];

/* The dynamic array. For each field and each letter in the pyramid
 * the number of occurences are kept.*/
int store[MAXSIZE][MAXSIZE][NUMLETTERS];

/* Best match. */
int max;
/* Best corresponding letter.*/
char best_char;

/* Number of test cases. */
int n;
/* Height of the pyramid. */
int size;

/* File pointer. */
FILE* fd_in;
FILE* fd_out;

/* Find the solution. */
void
calculatesolution()
{
	int i, j, k;
	/* The rows of the top neighbours. */
	int rleft[NUMLETTERS];
	int rright[NUMLETTERS];

	/* Best match. */
	max = 0;
	/* Clean the dynamic table. */
	memset(store[0][0], 0, sizeof(int) * NUMLETTERS);

	/* Initialize the store with the letter at the
	 * top of the pyramid. */
	store[0][0][letters[0][0] - 'A']++;

	/* For the other rows repeat this. */
	for (i = 1; i < size; i++) {
		for (j = 0; j <= i; j++) {
			/* If not on the edge get rleft, otherwise
			 * it's filled with zeroes.
			 */
			if (j > 0)
				memcpy(rleft, store[i - 1][j - 1],
						sizeof(int) * NUMLETTERS);
			else
				memset(rleft, 0, sizeof(int) * NUMLETTERS);
			/* If not on the other edge, get rright, otherwise
			 * it is filled with zeroes.
			 */
			if (j < size)
				memcpy(rright, store[i - 1][j],
						sizeof(int) * NUMLETTERS);
			else
				memset(rright, 0, sizeof(int) * NUMLETTERS);

			/* Find the maximum from the left and right values. */
			for (k = 0; k < NUMLETTERS; k++) {
				store[i][j][k] = (rleft[k] > rright[k]
						? rleft[k] : rright[k]);
			}
			/* And update the array with the corresponding leter
			 * of this position.
			 */
			store[i][j][letters[i][j] - 'A']++;
		}
	}

	/* After that's done, find the best match, because
	 * the outermost loop loops over the letters, we'll
	 * always find the lowest match in the alphabet.
	 */
	for (j = 0; j < NUMLETTERS; j++) {
		for (i = 0; i < size; i++) {
			if (store[size - 1][i][j] > max) {
				max = store[size - 1][i][j];
				best_char = j + 'A';
			}
		}
	}
}

/* Read the number of test cases. */
void
readinput_n(void)
{
	fd_in = fopen(INPUT, "r");
	if (fd_in == NULL)
		errx(EXIT_FAILURE, "could not open %s", OUTPUT);
	fscanf(fd_in, "%d\n", &n);
}

/* Read a test case. */
void
readinput(void)
{
	int i, j;
	fscanf(fd_in, "%d\n", &size);
	for (i = 0; i < size; i++) {
		for (j = 0; j <= i; j++)
			fscanf(fd_in, "%c", &letters[i][j]);
		fscanf(fd_in, "\n");
	}
}

/* Close the input. */
void
closeinput(void)
{
	fclose(fd_in);
}

/* Open the output. */
void
openoutput(void)
{
	fd_out = fopen(OUTPUT, "w");
	if (fd_out == NULL)
		errx(EXIT_FAILURE, "could not open %s", OUTPUT);
}

/* Write a solution to a test case. */
void
writeoutput(void)
{
	fprintf(fd_out, "%c %d\n", best_char, max);
}

/* Close the output. */
void
closeoutput(void)
{
	fclose(fd_out);
}

int
main(void)
{
	/* Read the number of test cases. */
	readinput_n();
	openoutput();
	/* Then for each test case. */
	while (n--) {
		readinput();
		calculatesolution();
		writeoutput();
	}
	/* And done. */
	closeinput();
	closeoutput();
	return 0;
}
