#include <err.h>
#include <stdio.h>
#include <stdlib.h>

#define INPUT "opgave7.in"
#define OUTPUT "opgave7.out"

#define MAXTIMES 1000000

/* Strategy:
 * It's basically brute force, Quick Sort will work fast enough.
 * Get all the times, use +1 for arrival, -1 for departure.
 *
 * Sort 'm by time. Sum the other column, and check while summing
 * for a maximum, but keep in mind that there's only a maximum at
 * a `time boundary', i.e. if this is the input:
 *
 * [already 10 people inside, max = 10]
 * 3 4 -> 3 +1
 * 1 3 -> 3 -1
 * 2 3 -> 3 -1
 *
 * Then, at time == 3 the maximum is not broken, because we first
 * have to substract the other two values.
 */

/* File pointers. */
FILE *fd_in;
FILE *fd_out;

/* Number of time entries in the file. */
int num_times;
/* Maximum load. */
int maximum;

/* Number of testcases. */
int n;

/* Number of times, need one departure, one arrival and + 1 for the boundary.
 */
int times[MAXTIMES*2 + 1][2];

/* Swaps two values. */
void
swap(int i, int j)
{
	int t[2];
	t[0] = times[i][0];
	t[1] = times[i][1];
	times[i][0] = times[j][0];
	times[i][1] = times[j][1];
	times[j][0] = t[0];
	times[j][1] = t[1];
}

/* Basic quick sort */
void
sort(int left, int right)
{
	int i, last;

	if (left >= right)
		return;

	swap(left, (left + right) >> 1);
	last = left;

	for (i = left + 1; i <= right; i++) {
		if (times[i][0] < times[left][0])
			swap(++last, i);
	
	}
	swap(left, last);
	sort(left, last - 1);
	sort(last + 1, right);
}

void
calculatesolution()
{
	int i;
	int load;

	/* Start empty. */
	load = 0;
	maximum = 0;
	/* Quick sort everything. */
	sort(0, num_times - 1);
	
	/* Start summing. */
	for (i = 0; i < num_times; i++) {
		load += times[i][1];
		if (load > maximum && times[i][0] != times[i + 1][0])
			maximum = load;
	}
}

/* Read the number of test cases. */
void
readinput_n(void)
{
	fd_in = fopen(INPUT, "r");
	if (fd_in == NULL)
		errx(EXIT_FAILURE, "could not open %s", INPUT);
	fscanf(fd_in, "%d\n", &n);
}

/* Read one test case. */
void
readinput(void)
{
	int i;
	int arrival;
	int departure;
	int n_times;
	num_times = 0;
	fscanf(fd_in, "%d\n", &n_times);
	printf("num times: %d\n", n_times);
	for (i = 0; i < n_times; i++) {
		fscanf(fd_in, "%d %d\n", &arrival, &departure);
		times[num_times][0] = arrival;
		times[num_times++][1] = 1;
		times[num_times][0] = departure;
		times[num_times++][1] = -1;
	}
	/* Explicit time boundary. */
	times[num_times][0] = -1;
}

/* Close the input. */
void
closeinput(void)
{
	fclose(fd_in);
}


/* Open the output. */
void
openoutput(void)
{
	fd_out = fopen(OUTPUT, "w");
	if (fd_out == NULL)
		errx(EXIT_FAILURE, "could not open %s", OUTPUT);
}


void
writeoutput(void)
{
	fprintf(fd_out, "%d\n", maximum);
}

/* Close the output. */
void
closeoutput(void)
{
	fclose(fd_out);
}

int
main(void)
{
	/* Read the number of test cases. */
	readinput_n();
	openoutput();
	/* Then for each test case. */
	while (n--) {
		readinput();
		calculatesolution();
		writeoutput();
	}
	/* And done. */
	closeinput();
	closeoutput();
	return 0;
}
